@@ -7,11 +7,19 @@ from sqlglot.tokens import Tokenizer, TokenType
 
 class ClickHouse(Dialect):
     normalize_functions = None
+    null_ordering = "nulls_are_last"
 
     class Tokenizer(Tokenizer):
         KEYWORDS = {
             **Tokenizer.KEYWORDS,
+            "NULLABLE": TokenType.NULLABLE,
             "FINAL": TokenType.FINAL,
+            "INT8": TokenType.TINYINT,
+            "INT16": TokenType.SMALLINT,
+            "INT32": TokenType.INT,
+            "INT64": TokenType.BIGINT,
+            "FLOAT32": TokenType.FLOAT,
+            "FLOAT64": TokenType.DOUBLE,
         }
 
     class Parser(Parser):
@@ -24,6 +32,8 @@ class ClickHouse(Dialect):
             return this
 
     class Generator(Generator):
+        STRUCT_DELIMITER = ("(", ")")
+
         TRANSFORMS = {
             **Generator.TRANSFORMS,
             exp.Final: lambda self, e: f"{self.sql(e, 'this')} FINAL",
