@@ -52,20 +52,32 @@ class _Dialect(type):
         klass.parser_class = getattr(klass, "Parser", Parser)
         klass.generator_class = getattr(klass, "Generator", Generator)
 
+        identifiers = dict(
+            (identifier[0], identifier[1])
+            if isinstance(identifier, tuple)
+            else (identifier, identifier)
+            for identifier in klass.identifiers
+        )
+        klass.identifier_start, klass.identifier_end = list(identifiers.items())[0]
+
         klass.tokenizer = klass.tokenizer_class(
-            identifier=klass.identifier,
+            identifiers=identifiers,
             escape=klass.escape,
         )
+
         return klass
 
 
 class Dialect(metaclass=_Dialect):
-    identifier = None
+    identifiers = ['"']
+    identifier_start = '"'
+    identifier_end = '"'
     escape = "'"
     index_offset = 0
     unnest_column_only = False
     alias_post_tablesample = False
     normalize_functions = "upper"
+    null_ordering = "nulls_are_small"
 
     date_format = "'%Y-%m-%d'"
     dateint_format = "'%Y%m%d'"
@@ -131,6 +143,7 @@ class Dialect(metaclass=_Dialect):
                 "index_offset": self.index_offset,
                 "unnest_column_only": self.unnest_column_only,
                 "alias_post_tablesample": self.alias_post_tablesample,
+                "null_ordering": self.null_ordering,
                 **opts,
             },
         )
@@ -140,7 +153,8 @@ class Dialect(metaclass=_Dialect):
         return self.generator_class(
             **{
                 "quote": self.tokenizer_class.QUOTES[0],
-                "identifier": self.identifier,
+                "identifier_start": self.identifier_start,
+                "identifier_end": self.identifier_end,
                 "escape": self.escape,
                 "index_offset": self.index_offset,
                 "time_mapping": self.inverse_time_mapping,
@@ -148,6 +162,7 @@ class Dialect(metaclass=_Dialect):
                 "unnest_column_only": self.unnest_column_only,
                 "alias_post_tablesample": self.alias_post_tablesample,
                 "normalize_functions": self.normalize_functions,
+                "null_ordering": self.null_ordering,
                 **opts,
             }
         )
@@ -220,16 +235,25 @@ def no_trycast_sql(self, expression):
 
 def struct_extract_sql(self, expression):
     this = self.sql(expression, "this")
-    struct_key = self.sql(expression, "expression").replace(self.quote, self.identifier)
+    struct_key = self.sql(exp.Identifier(this=expression.expression, quoted=True))
     return f"{this}.{struct_key}"
 
 
-def format_time_lambda(exp_class, dialect, default=False):
+def format_time_lambda(exp_class, dialect, default=None):
+    """Helper used for time expressions.
+
+    Args
+        exp_class (Class): the expression class to instantiate
+        dialect (string): sql dialect
+        default (Option[bool | str]): the default format, True being time
+    """
+
     def _format_time(args):
         return exp_class(
             this=list_get(args, 0),
             format=Dialect[dialect].format_time(
-                list_get(args, 1) or (Dialect[dialect].time_format if default else None)
+                list_get(args, 1)
+                or (Dialect[dialect].time_format if default is True else default)
             ),
         )
 
