@@ -97,14 +97,17 @@ def _str_position_sql(self, expression):
     return f"STRPOS({this}, {substr})"
 
 
-def _ts_or_ds_to_date_str_sql(self, expression):
-    this = self.sql(expression, "this")
-    return f"DATE_FORMAT(DATE_PARSE(SUBSTR(CAST({this} AS VARCHAR), 1, 10), {Presto.date_format}), {Presto.date_format})"
+def _str_to_time_sql(self, expression):
+    return f"DATE_PARSE({self.sql(expression, 'this')}, {self.format_time(expression)})"
 
 
 def _ts_or_ds_to_date_sql(self, expression):
-    this = self.sql(expression, "this")
-    return f"CAST(DATE_PARSE(SUBSTR(CAST({this} AS VARCHAR), 1, 10), {Presto.date_format}) AS DATE)"
+    time_format = self.format_time(expression)
+    if time_format and time_format not in (Presto.time_format, Presto.date_format):
+        return f"CAST({_str_to_time_sql(self, expression)} AS DATE)"
+    return (
+        f"CAST(SUBSTR(CAST({self.sql(expression, 'this')} AS VARCHAR), 1, 10) AS DATE)"
+    )
 
 
 def _ts_or_ds_add_sql(self, expression):
@@ -116,6 +119,7 @@ def _ts_or_ds_add_sql(self, expression):
 
 class Presto(Dialect):
     index_offset = 1
+    null_ordering = "nulls_are_last"
     time_format = "'%Y-%m-%d %H:%i:%S'"
     time_mapping = MySQL.time_mapping
 
@@ -149,6 +153,7 @@ class Presto(Dialect):
         }
 
     class Generator(Generator):
+
         STRUCT_DELIMITER = ("(", ")")
 
         WITH_PROPERTIES = [
@@ -201,7 +206,8 @@ class Presto(Dialect):
             exp.Schema: _schema_sql,
             exp.SortArray: _no_sort_array,
             exp.StrPosition: _str_position_sql,
-            exp.StrToTime: lambda self, e: f"DATE_PARSE({self.sql(e, 'this')}, {self.format_time(e)})",
+            exp.StrToDate: lambda self, e: f"CAST({_str_to_time_sql(self, e)} AS DATE)",
+            exp.StrToTime: _str_to_time_sql,
             exp.StrToUnix: lambda self, e: f"TO_UNIXTIME(DATE_PARSE({self.sql(e, 'this')}, {self.format_time(e)}))",
             exp.StructExtract: struct_extract_sql,
             exp.TableFormatProperty: lambda self, e: f"TABLE_FORMAT = '{e.text('value').upper()}'",
@@ -213,7 +219,6 @@ class Presto(Dialect):
             exp.TimeToUnix: rename_func("TO_UNIXTIME"),
             exp.TsOrDiToDi: lambda self, e: f"CAST(SUBSTR(REPLACE(CAST({self.sql(e, 'this')} AS VARCHAR), '-', ''), 1, 8) AS INT)",
             exp.TsOrDsAdd: _ts_or_ds_add_sql,
-            exp.TsOrDsToDateStr: _ts_or_ds_to_date_str_sql,
             exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
             exp.UnixToStr: lambda self, e: f"DATE_FORMAT(FROM_UNIXTIME({self.sql(e, 'this')}), {self.format_time(e)})",
             exp.UnixToTime: rename_func("FROM_UNIXTIME"),
