@@ -17,19 +17,24 @@ def _check_int(s):
 def _snowflake_to_timestamp(args):
     if len(args) == 2:
         first_arg, second_arg = args
-        if first_arg.is_string:
+        if second_arg.is_string:
             # case: <string_expr> [ , <format> ]
             return format_time_lambda(exp.StrToTime, "snowflake")(args)
 
         # case: <numeric_expr> [ , <scale> ]
-        if second_arg.this not in ["0", "3", "9"]:
+        if second_arg.name not in ["0", "3", "9"]:
             raise ValueError(
                 f"Scale for snowflake numeric timestamp is {second_arg}, but should be 0, 3, or 9"
             )
 
-        timestamp = int(first_arg.name)
-        scale = int(second_arg.name)
-        return exp.UnixToTime(this=str(timestamp // (10**scale)))
+        if second_arg.name == "0":
+            timescale = exp.UnixToTime.SECONDS
+        elif second_arg.name == "3":
+            timescale = exp.UnixToTime.MILLIS
+        elif second_arg.name == "9":
+            timescale = exp.UnixToTime.MICROS
+
+        return exp.UnixToTime(this=first_arg, scale=timescale)
 
     first_arg = list_get(args, 0)
     if not isinstance(first_arg, Literal):
@@ -48,7 +53,21 @@ def _snowflake_to_timestamp(args):
     return exp.UnixToTime.from_arg_list(args)
 
 
+def _unix_to_time(self, expression):
+    scale = expression.args.get("scale")
+    timestamp = self.sql(expression, "this")
+    if scale in [None, exp.UnixToTime.SECONDS]:
+        return f"TO_TIMESTAMP({timestamp})"
+    if scale == exp.UnixToTime.MILLIS:
+        return f"TO_TIMESTAMP({timestamp}, 3)"
+    if scale == exp.UnixToTime.MICROS:
+        return f"TO_TIMESTAMP({timestamp}, 9)"
+
+    raise ValueError("Improper scale for timestamp")
+
+
 class Snowflake(Dialect):
+    null_ordering = "nulls_are_large"
     time_format = "'yyyy-mm-dd hh24:mi:ss'"
 
     # pylint: disable=duplicate-code
@@ -86,6 +105,7 @@ class Snowflake(Dialect):
     class Parser(Parser):
         FUNCTIONS = {
             **Parser.FUNCTIONS,
+            "ARRAYAGG": exp.ArrayAgg.from_arg_list,
             "IFF": exp.If.from_arg_list,
             "TO_TIMESTAMP": _snowflake_to_timestamp,
         }
@@ -111,5 +131,15 @@ class Snowflake(Dialect):
             **Generator.TRANSFORMS,
             exp.If: rename_func("IFF"),
             exp.StrToTime: lambda self, e: f"TO_TIMESTAMP({self.sql(e, 'this')}, {self.format_time(e)})",
-            exp.UnixToTime: rename_func("TO_TIMESTAMP"),
+            exp.UnixToTime: _unix_to_time,
         }
+
+        def except_op(self, expression):
+            if not expression.args.get("distinct", False):
+                self.unsupported("EXCEPT with All is not supported in Snowflake")
+            return super().except_op(expression)
+
+        def intersect_op(self, expression):
+            if not expression.args.get("distinct", False):
+                self.unsupported("INTERSECT with All is not supported in Snowflake")
+            return super().intersect_op(expression)
