@@ -1,7 +1,7 @@
 import logging
 
 from sqlglot import exp
-from sqlglot.errors import ErrorLevel, UnsupportedError
+from sqlglot.errors import ErrorLevel, UnsupportedError, concat_errors
 from sqlglot.helper import apply_index_offset, csv, ensure_list
 from sqlglot.time import format_time
 from sqlglot.tokens import TokenType, Tokenizer
@@ -19,7 +19,8 @@ class Generator:
             represents a python time format and the output the target time format
         time_trie (trie): a trie of the time_mapping keys
         pretty (bool): if set to True the returned string will be formatted. Default: False.
-        identifier (str): specifies which character to use to delimit identifiers. Default: ".
+        identifier_start (str): specifies which starting character to use to delimit identifiers. Default: ".
+        identifier_end (str): specifies which ending character to use to delimit identifiers. Default: ".
         identify (bool): if set to True all identifiers will be delimited by the corresponding
             character.
         normalize (bool): if set to True all identifiers will lower cased
@@ -35,6 +36,12 @@ class Generator:
             Default: False
         unsupported_level (ErrorLevel): determines the generator's behavior when it encounters
             unsupported expressions. Default ErrorLevel.WARN.
+        null_ordering (str): Indicates the default null ordering method to use if not explicitly set.
+            Options are "nulls_are_small", "nulls_are_large", "nulls_are_last".
+            Default: "nulls_are_small"
+        max_unsupported (int): Maximum number of unsupported messages to include in a raised UnsupportedError.
+            This is only relevant if unsupported_level is ErrorLevel.RAISE.
+            Default: 3
     """
 
     TRANSFORMS = {
@@ -53,6 +60,8 @@ class Generator:
         exp.TsOrDsAdd: lambda self, e: f"TS_OR_DS_ADD({self.sql(e, 'this')}, {self.sql(e, 'expression')}, {self.sql(e, 'unit')})",
     }
 
+    NULL_ORDERING_SUPPORTED = True
+
     TYPE_MAPPING = {}
 
     TOKEN_MAPPING = {}
@@ -78,7 +87,8 @@ class Generator:
         "time_trie",
         "pretty",
         "configured_pretty",
-        "identifier",
+        "identifier_start",
+        "identifier_end",
         "identify",
         "normalize",
         "quote",
@@ -90,6 +100,8 @@ class Generator:
         "normalize_functions",
         "unsupported_level",
         "unsupported_messages",
+        "null_ordering",
+        "max_unsupported",
         "_indent",
     )
 
@@ -98,7 +110,8 @@ class Generator:
         time_mapping=None,
         time_trie=None,
         pretty=None,
-        identifier=None,
+        identifier_start=None,
+        identifier_end=None,
         identify=False,
         normalize=False,
         quote=None,
@@ -110,6 +123,8 @@ class Generator:
         alias_post_tablesample=False,
         normalize_functions="upper",
         unsupported_level=ErrorLevel.WARN,
+        null_ordering=None,
+        max_unsupported=3,
     ):
         # pylint: disable=too-many-arguments
         import sqlglot
@@ -118,7 +133,8 @@ class Generator:
         self.time_trie = time_trie
         self.pretty = pretty if pretty is not None else sqlglot.pretty
         self.configured_pretty = self.pretty
-        self.identifier = identifier or '"'
+        self.identifier_start = identifier_start or '"'
+        self.identifier_end = identifier_end or '"'
         self.identify = identify
         self.normalize = normalize
         self.quote = quote or "'"
@@ -130,6 +146,8 @@ class Generator:
         self.normalize_functions = normalize_functions
         self.unsupported_level = unsupported_level
         self.unsupported_messages = []
+        self.max_unsupported = max_unsupported
+        self.null_ordering = null_ordering
         self._indent = indent
 
     def generate(self, expression):
@@ -148,14 +166,19 @@ class Generator:
         if self.unsupported_level == ErrorLevel.IGNORE:
             return sql
 
-        for msg in self.unsupported_messages:
-            if self.unsupported_level == ErrorLevel.RAISE:
-                raise UnsupportedError(msg)
-            logger.warning(msg)
+        if self.unsupported_level == ErrorLevel.WARN:
+            for msg in self.unsupported_messages:
+                logger.warning(msg)
+        elif self.unsupported_level == ErrorLevel.RAISE and self.unsupported_messages:
+            raise UnsupportedError(
+                concat_errors(self.unsupported_messages, self.max_unsupported)
+            )
 
         return sql
 
     def unsupported(self, message):
+        if self.unsupported_level == ErrorLevel.IMMEDIATE:
+            raise UnsupportedError(message)
         self.unsupported_messages.append(message)
 
     def sep(self, sep=" "):
@@ -352,12 +375,12 @@ class Generator:
     def except_sql(self, expression):
         return self.prepend_ctes(
             expression,
-            self.set_operation(
-                expression,
-                f"EXCEPT{' DISTINCT' if expression.args.get('distinct') else ''}",
-            ),
+            self.set_operation(expression, self.except_op(expression)),
         )
 
+    def except_op(self, expression):
+        return f"EXCEPT{'' if expression.args.get('distinct') else ' ALL'}"
+
     def fetch_sql(self, expression):
         direction = expression.args.get("direction")
         direction = f" {direction.upper()}" if direction else ""
@@ -379,7 +402,7 @@ class Generator:
         value = expression.name
         value = value.lower() if self.normalize else value
         if expression.args.get("quoted") or self.identify:
-            return f"{self.identifier}{value}{self.identifier}"
+            return f"{self.identifier_start}{value}{self.identifier_end}"
         return value
 
     def partition_sql(self, expression):
@@ -453,12 +476,12 @@ class Generator:
     def intersect_sql(self, expression):
         return self.prepend_ctes(
             expression,
-            self.set_operation(
-                expression,
-                f"INTERSECT{' DISTINCT' if expression.args.get('distinct') else ''}",
-            ),
+            self.set_operation(expression, self.intersect_op(expression)),
         )
 
+    def intersect_op(self, expression):
+        return f"INTERSECT{'' if expression.args.get('distinct') else ' ALL'}"
+
     def table_sql(self, expression):
         return ".".join(
             part
@@ -607,18 +630,37 @@ class Generator:
 
     def ordered_sql(self, expression):
         desc = expression.args.get("desc")
-        desc = " DESC" if desc else ""
-        return f"{self.sql(expression, 'this')}{desc}"
+        asc = not desc
+        nulls_first = expression.args.get("nulls_first")
+        nulls_last = not nulls_first
+        nulls_are_large = self.null_ordering == "nulls_are_large"
+        nulls_are_small = self.null_ordering == "nulls_are_small"
+        nulls_are_last = self.null_ordering == "nulls_are_last"
 
-    def select_sql(self, expression):
-        hint = self.sql(expression, "hint")
-        distinct = self.sql(expression, "distinct")
-        distinct = f" {distinct}" if distinct else ""
-        expressions = self.expressions(expression)
-        sep = self.sep() if expressions else ""
-        sql = csv(
-            f"SELECT{hint}{distinct}{sep}{expressions}",
-            self.sql(expression, "from"),
+        sort_order = " DESC" if desc else ""
+        nulls_sort_change = ""
+        if nulls_first and (
+            (asc and nulls_are_large) or (desc and nulls_are_small) or nulls_are_last
+        ):
+            nulls_sort_change = " NULLS FIRST"
+        elif (
+            nulls_last
+            and ((asc and nulls_are_small) or (desc and nulls_are_large))
+            and not nulls_are_last
+        ):
+            nulls_sort_change = " NULLS LAST"
+
+        if nulls_sort_change and not self.NULL_ORDERING_SUPPORTED:
+            self.unsupported(
+                "Sorting in an ORDER BY on NULLS FIRST/NULLS LAST is not supported by this dialect"
+            )
+            nulls_sort_change = ""
+
+        return f"{self.sql(expression, 'this')}{sort_order}{nulls_sort_change}"
+
+    def query_modifiers(self, expression, *sqls):
+        return csv(
+            *sqls,
             *[self.sql(sql) for sql in expression.args.get("laterals", [])],
             *[self.sql(sql) for sql in expression.args.get("joins", [])],
             self.sql(expression, "where"),
@@ -634,6 +676,18 @@ class Generator:
             self.sql(expression, "offset"),
             sep="",
         )
+
+    def select_sql(self, expression):
+        hint = self.sql(expression, "hint")
+        distinct = self.sql(expression, "distinct")
+        distinct = f" {distinct}" if distinct else ""
+        expressions = self.expressions(expression)
+        expressions = f"{self.sep()}{expressions}" if expressions else expressions
+        sql = self.query_modifiers(
+            expression,
+            f"SELECT{hint}{distinct}{expressions}",
+            self.sql(expression, "from"),
+        )
         return self.prepend_ctes(expression, sql)
 
     def schema_sql(self, expression):
@@ -658,17 +712,10 @@ class Generator:
     def subquery_sql(self, expression):
         alias = self.sql(expression, "alias")
 
-        return csv(
+        return self.query_modifiers(
+            expression,
             self.wrap(expression),
             f" AS {alias}" if alias else "",
-            *[self.sql(sql) for sql in expression.args.get("joins", [])],
-            self.sql(expression, "distribute"),
-            self.sql(expression, "sort"),
-            self.sql(expression, "cluster"),
-            self.sql(expression, "order"),
-            self.sql(expression, "limit"),
-            self.sql(expression, "offset"),
-            sep="",
         )
 
     def qualify_sql(self, expression):
@@ -678,11 +725,12 @@ class Generator:
     def union_sql(self, expression):
         return self.prepend_ctes(
             expression,
-            self.set_operation(
-                expression, f"UNION{'' if expression.args.get('distinct') else ' ALL'}"
-            ),
+            self.set_operation(expression, self.union_op(expression)),
         )
 
+    def union_op(self, expression):
+        return f"UNION{'' if expression.args.get('distinct') else ' ALL'}"
+
     def unnest_sql(self, expression):
         args = self.expressions(expression, flat=True)
         alias = expression.args.get("alias")
@@ -798,13 +846,18 @@ class Generator:
 
     def in_sql(self, expression):
         query = expression.args.get("query")
-        in_sql = (
-            self.wrap(query)
-            if query
-            else f"({self.expressions(expression, flat=True)})"
-        )
+        unnest = expression.args.get("unnest")
+        if query:
+            in_sql = self.wrap(query)
+        elif unnest:
+            in_sql = self.in_unnest_op(unnest)
+        else:
+            in_sql = f"({self.expressions(expression, flat=True)})"
         return f"{self.sql(expression, 'this')} IN {in_sql}"
 
+    def in_unnest_op(self, unnest):
+        return f"(SELECT {self.sql(unnest)})"
+
     def interval_sql(self, expression):
         return f"INTERVAL {self.sql(expression, 'this')} {self.sql(expression, 'unit')}"
 
@@ -1006,8 +1059,9 @@ class Generator:
     def set_operation(self, expression, op):
         this = self.sql(expression, "this")
         op = self.seg(op)
-        expression = self.sql(expression, "expression")
-        return f"{this}{op}{self.sep()}{expression}"
+        return self.query_modifiers(
+            expression, f"{this}{op}{self.sep()}{self.sql(expression, 'expression')}"
+        )
 
     def token_sql(self, token_type):
         return self.TOKEN_MAPPING.get(token_type, token_type.name)
